ccc  m2range.f    Time-stamp: <2005-08-17 14:05:31 hamanatk>
c
c   WAHT IS THIS ?
c   This program computes the 68% confidence interval of the PDF(M2) as 
c   a function of the redshift (z2).
c   PDF(M2) is the normalized conditional mass function 
c   derived from the extended PS model,
c   PDF(M2) \propto n(M2,z2|M1,z1),
c   with n(M2,z2|M1,z1)=P(M2,z2|M1,z1)/M2.
c   Here P is the conditional probability that a material in a halo of mass M1
c   at z1 will be in a halo of mass M2 ($M2>M1) at a later redshift z2.
c
c   USAGE:
c   (1) compile this with a fortran compiler
c       example: g77/ifort/ifc/f90/f77 -O2 m2range.f -o m2range.exe
c       Notice that this does not work for some fortran compilers,
c       the recommended compiler is ifort wiht an intel CPU or the latest g77.
c   (2) set parameters in  "m2range.input"
c      (see "m2range.input" for details)
c   (3) execute 
c       m2range.exe < m2range.input
c   (*) If it does not work, try a complile option that
c       re-defines REAL as DOUBLE PRECISION (REAL*8) such like
c       -r8, -dbl or -Ad.
c   (**) For most of modern computers, it takes a few minutes at longest.
c       If your job takes longer time and does not finish, you'd be better
c       to try the other compiler.
c   (***) Valid for flat model (Omega_m + Omega_lambda =1) only
c
c   HISTORY:
c   2005/08/17 --- by Takashi Hamana
c
c   Bug report:
c   hamanatk[at]cc.nao.ac.jp
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /omegas/ omem,omev
      common /powerspec/ gfin,atps
      
      character*99 fname
      
      parameter (ch0=3000.0)
      parameter (mmin=800,mmax=1800)
      parameter (delm=0.01)
      parameter (mm=mmax-mmin+1)

      dimension dp(mm)
      dimension xx(mm)

ccc useful numbers
      pi=4.0*atan(1.0)
      tps=2.0*pi**2
      sqtp=sqrt(2.0*pi)
      
      onethird=1.0/3.0
      e2ten=2.30259 !ln(x)=ln(10)*log(x), ln(10)=e2ten
      rhocr=2.7755e11 ! M_sun h^2 /Mpc^3

ccc flags
      igrfrag=-1
      idlogsdlogm=-1

ccc cosmology
      read(*,*)hubp
      read(*,*)omem
      read(*,*)omev
      read(*,*)omeb
      read(*,*)sig8

      ok=abs(1.0-omem-omev)
      if (ok.gt.1.0e-3) pause 'flat model only'
      omebh2=omeb*hubp**2
      rho0=omem*rhocr
      r2m=4.0*pi*rho0/3.0
      
      read(*,'(a99)')fname

ccc Gamma factor
      gfin=exp(omeb+sqrt(2.0*hubp)*omeb/omem)/
     $     (omem*hubp*ch0)

ccc normalization
      call normPk(anrm,sig8)
      atps=anrm/tps

ccc set simga^2-M relation at z=0
      call dlogsdlogm(idlogsdlogm,r2m,0.0,0.0)

ccc early epoch halo
      read(*,*)z1,hm1

ccc early epoch halo; z1, M1, t1 (t2>t1)
      a1=1.0/(1.0+z1)
      
      nz=int((1.0-a1)*100)

ccc growth rate at z1
      growth1=gr(a1,igrfrag)
      ga1=growth1*a1
ccc delta_c at z1
      dc1=deltac(a1,omem,pi)/ga1
ccc variance
      r1=(hm1/r2m)**onethird
      var1=var0(r1)
      
ccc z-interval
      dela=(1.0-a1)/nz

 101  format(a6,1pe13.5)
 102  format(a28)
      open(10,file=fname,status='unknown')
      write(10,101)'# z1 =',z1
      write(10,101)'# M1 =',hm1
      write(10,102)'# col-1: z                  '
      write(10,102)'# col-2: the scale factor   '
      write(10,102)'# col-3: <M>                '
      write(10,102)'# col-4: mode of n(M)       '
      write(10,102)'# col-5: lower 68.3% of n(M)'
      write(10,102)'# col-6: upper 68.3% of n(M)'
      write(10,102)'#==========================#'
      write(10,'(6(1pe13.5))')z1,a1,hm1,hm1,hm1,hm1
      do j=1,nz
ccc later epoch halo; z2, M2, t2 (t2>t1)
         a2=a1+j*dela
         z2=1.0/a2-1.0
c         z2=z1-j*delz
c         a2=1.0/(1.0+z2)
ccc growth rate at z2
         growth2=gr(a2,igrfrag)
         ga2=growth2*a2
ccc delta_c at z2
         dc2=deltac(a2,omem,pi)/ga2
      
ccc mass function
         dc12=dc1-dc2
         do i=1,mm
            dp(i)=0.0
         enddo
         acum=0.0
         bun=0.0
         nn=0
         do i=mmin,mmax
            nn=nn+1
            xx(nn)=i*1.0
            hm2=10.0**(i*delm)
            r2=(hm2/r2m)**onethird
            var2=var0(r2)
            if (var1.le.var2) cycle
            call dlogsdlogm(idlogsdlogm,r2m,hm1,ss)
            var12=var1-var2
            p2=dc12*dc2*ss*(var1/var12)**1.5
     $           *exp(-0.5*(var2*dc1-var1*dc2)**2/(var1*var2*var12))
     $           /(sqtp*dc1*sqrt(var2)*hm2)
            p2log10=p2*e2ten*delm
            bun=bun+p2log10
            acum=acum+p2log10*hm2
            dp(nn)=p2log10
            ddp=dp(nn)-dp(nn-1)
            if (ddp.lt.0.0) xx(nn)=-1.0*xx(nn)
         enddo
         am=acum/bun

         do i=1,mm
            dp(i)=dp(i)/bun
         enddo

         call sort2(mm,dp,xx)

         s0=10.0**(xx(mm)*delm)

         tp=0.0
         do i=mm,1,-1
            tp=tp+dp(i)
            if (tp.ge.0.683) then
               sig=xx(i)
               k=i
               l=i
 77            l=l+1
               if (l.gt.mm) goto 777
               if (sig*xx(l).gt.0.0) then
                  x=abs(xx(l))+delx*(0.683-dp(l))/(dp(i)-dp(l))
                  x=10.0**(x*delm)
               else
                  goto 77
               endif
               if (xx(i).gt.0.0) then
                  s1=x
               else
                  s2=x
               endif
               exit                  
            endif
         enddo

         do i=k,1,-1
            if (sig*xx(i).lt.0.0) then
               sig2=xx(i)
               l=i
 88            l=l+1
               if (sig2*xx(l).gt.0.0) then
                  x=abs(xx(l))+delx*(0.683-dp(l))/(dp(i)-dp(l))
                  x=10.0**(x*delm)
               else
                  goto 88
               endif
               if (xx(i).gt.0.0) then
                  s1=x
               else
                  s2=x
               endif
               exit
            endif
         enddo
         write(10,'(6(1pe13.5))')z2,a2,am,s0,s1,s2
 777  enddo
      close(10)
        
      end


ccc dlogsigma^2/dlogM
      subroutine dlogsdlogm(idlogsdlogm,r2m,hm,ss)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      save sigma2,hmass,dls2dlm,hmmin,hmmax

      parameter (mmin=50,mmax=450,delm=0.1,mall=mmax-mmin+1)
      parameter (onethird=1.0/3.0)
      dimension sigma2(mall),hmass(mall),dls2dlm(mall)

      if (idlogsdlogm.eq.-1) then
         do i=1,mall
            ii=i+mmin-1
            hmass(i)=delm*ii
            x=exp(hmass(i))
            r=(x/r2m)**onethird
            sigma2(i)=log(var0(r))
         enddo
         
         dls2dlm(1)=abs((sigma2(2)-sigma2(1))/(hmass(2)-hmass(1)))
         do i=2,mall-1
            dls2dlm(i)=abs((sigma2(i+1)-sigma2(i-1))/
     $           (hmass(i+1)-hmass(i-1)))
         enddo
         dls2dlm(mall)=abs((sigma2(mall)-sigma2(mall-1))/
     $        (hmass(mall)-hmass(mall-1)))
         
         hmmin=hmass(1)
         hmmax=hmass(mall)

         idlogsdlogm=1
      else
         xx=log(hm)
         if (xx.lt.hmmin) pause 'halo mass less than min'
         if (xx.gt.hmmax) pause 'halo mass greater than max'

         call hunt(hmass,mall,xx,j)
         
         ds=(dls2dlm(j+1)-dls2dlm(j))/(hmass(j+1)-hmass(j))
         ss=dls2dlm(j)+ds*(xx-hmass(j))

      endif

      return
      end

ccc simga^2(R)
      function var0(x)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /powerspec/ gfin,atps
      common /sigmar/ rsmooth
      external dvar_dr
      parameter (ch0=3000.0)
      parameter (wmin=-40.0,wmax=22.0)
      rsmooth=x/ch0
      call qromb(dvar_dr,wmin,wmax,ss)
      var0=atps*ss
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccc
      function dvar_dr(x)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /sigmar/ rsmooth
      common /powerspec/ gfin,atps
      w=exp(x)
      rk=rsmooth*w
      q=w*gfin
      dvar_dr=trnf2(q)*w**4*filtTH(rk)**2
      return
      end 

ccc normalization of power spectrum 
      subroutine normPk(a_norm,sig8)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      external delta
      parameter (wmin=-40.0,wmax=22.0)
      pi=4.0*atan(1.0)
      tps=2.0*pi**2
      call qromb(delta,wmin,wmax,seki)
      y=seki/tps
      a_norm=sig8*sig8/y
      return
      end
ccccccccc
      function delta(x)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /powerspec/ gfin,atps
      rn=8.0/3000.0
      w=exp(x)
      q=w*gfin
      r=w*rn
      filt=filtTH(r)
      tranf=trnf2(q)
      delta=w**4*filt*filt*tranf
      return
      end

ccccccccc Top-Hat filter
      function filtTH(r)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      filtTH=3.0*(sin(r)-r*cos(r))/r**3
      return
      end
ccc CDM transfer function
      function trnf2(q)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      qq=q*q
      trnf2=0.182628*log(1.0+2.34*q)**2/qq
     $     /sqrt(1.0+3.89*q+259.21*qq
     $     +162.77*qq*q+2027.17*qq*qq)
      return
      end

ccc delta_c ***VALID for only FLAT model***
      function deltac(a,omem,pi)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      x=a*(1.0/omem-1.0)**(1.0/3.0)
      deltac=3.0*(12.0*pi)**(2.0/3.0)/20.0*
     $     (1.0-0.00123*log10(1.0+x**3))
      return
      end


ccccccccccccccccccccccccccccccccccccccccccccccccccc
      function gr(a,igrfrag)
      implicit real (a-h,o-z)
      implicit integer (i-n)
      common /omegas/ omem,omev
      save grnorm
      if (igrfrag.eq.-1) then
         ot=omem
         vt=omev
         grnorm=0.4*(ot**(0.571428571)-vt+
     $        (1.0+0.5*ot)*(1.0+vt*0.014285714))/ot
         igrfrag=1
      endif
      a2=a*a
      et=1.0/(a*omem+a2*a2*omev)
      ot=omem*a*et
      vt=omev*et*a**4
      graw=2.5*ot/(ot**(0.571428571)-vt+
     $     (1.0+0.5*ot)*(1.0+vt*0.014285714))
      gr=graw*grnorm
      return
      end

ccc QROMB package
      SUBROUTINE qromb(func,a,b,ss)
      INTEGER JMAX,JMAXP,K,KM
      REAL a,b,func,ss,EPS
      EXTERNAL func
      PARAMETER (EPS=1.e-9, JMAX=40, JMAXP=JMAX+1, K=10, KM=K-1)
CU    USES polint,trapzd
      INTEGER j
      REAL dss,h(JMAXP),s(JMAXP)
      h(1)=1.
      do 11 j=1,JMAX
        call trapzd(func,a,b,s(j),j)
        if (j.ge.K) then
          call polint(h(j-KM),s(j-KM),K,0.,ss,dss)
          if (abs(dss).le.EPS*abs(ss)) return
        endif
        s(j+1)=s(j)
        h(j+1)=0.25*h(j)
11    continue
      pause 'too many steps in qromb'
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.
      SUBROUTINE polint(xa,ya,n,x,y,dy)
      INTEGER n,NMAX
      REAL dy,x,y,xa(n),ya(n)
      PARAMETER (NMAX=20)
      INTEGER i,m,ns
      REAL den,dif,dift,ho,hp,w,c(NMAX),d(NMAX)
      ns=1
      dif=abs(x-xa(1))
      do 11 i=1,n
        dift=abs(x-xa(i))
        if (dift.lt.dif) then
          ns=i
          dif=dift
        endif
        c(i)=ya(i)
        d(i)=ya(i)
11    continue
      y=ya(ns)
      ns=ns-1
      do 13 m=1,n-1
        do 12 i=1,n-m
          ho=xa(i)-x
          hp=xa(i+m)-x
          w=c(i+1)-d(i)
          den=ho-hp
          if(den.eq.0.)pause 'failure in polint'
          den=w/den
          d(i)=hp*den
          c(i)=ho*den
12      continue
        if (2*ns.lt.n-m)then
          dy=c(ns+1)
        else
          dy=d(ns)
          ns=ns-1
        endif
        y=y+dy
13    continue
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.
      SUBROUTINE trapzd(func,a,b,s,n)
      INTEGER n
      REAL a,b,s,func
      EXTERNAL func
      INTEGER it,j
      REAL del,sum,tnm,x
      if (n.eq.1) then
        s=0.5*(b-a)*(func(a)+func(b))
      else
        it=2**(n-2)
        tnm=it
        del=(b-a)/tnm
        x=a+0.5*del
        sum=0.
        do 11 j=1,it
          sum=sum+func(x)
          x=x+del
11      continue
        s=0.5*(s+(b-a)*sum/tnm)
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.

      SUBROUTINE hunt(xx,n,x,jlo)
      INTEGER jlo,n
      REAL x,xx(n)
      INTEGER inc,jhi,jm
      LOGICAL ascnd
      ascnd=xx(n).gt.xx(1)
      if(jlo.le.0.or.jlo.gt.n)then
        jlo=0
        jhi=n+1
        goto 3
      endif
      inc=1
      if(x.ge.xx(jlo).eqv.ascnd)then
1       jhi=jlo+inc
        if(jhi.gt.n)then
          jhi=n+1
        else if(x.ge.xx(jhi).eqv.ascnd)then
          jlo=jhi
          inc=inc+inc
          goto 1
        endif
      else
        jhi=jlo
2       jlo=jhi-inc
        if(jlo.lt.1)then
          jlo=0
        else if(x.lt.xx(jlo).eqv.ascnd)then
          jhi=jlo
          inc=inc+inc
          goto 2
        endif
      endif
3     if(jhi-jlo.eq.1)return
      jm=(jhi+jlo)/2
      if(x.gt.xx(jm).eqv.ascnd)then
        jlo=jm
      else
        jhi=jm
      endif
      goto 3
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.


      SUBROUTINE sort2(n,arr,brr)
      INTEGER n,M,NSTACK
      REAL arr(n),brr(n)
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,ir,j,jstack,k,l,istack(NSTACK)
      REAL a,b,temp
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 12 j=l+1,ir
          a=arr(j)
          b=brr(j)
          do 11 i=j-1,1,-1
            if(arr(i).le.a)goto 2
            arr(i+1)=arr(i)
            brr(i+1)=brr(i)
11        continue
          i=0
2         arr(i+1)=a
          brr(i+1)=b
12      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        temp=arr(k)
        arr(k)=arr(l+1)
        arr(l+1)=temp
        temp=brr(k)
        brr(k)=brr(l+1)
        brr(l+1)=temp
        if(arr(l+1).gt.arr(ir))then
          temp=arr(l+1)
          arr(l+1)=arr(ir)
          arr(ir)=temp
          temp=brr(l+1)
          brr(l+1)=brr(ir)
          brr(ir)=temp
        endif
        if(arr(l).gt.arr(ir))then
          temp=arr(l)
          arr(l)=arr(ir)
          arr(ir)=temp
          temp=brr(l)
          brr(l)=brr(ir)
          brr(ir)=temp
        endif
        if(arr(l+1).gt.arr(l))then
          temp=arr(l+1)
          arr(l+1)=arr(l)
          arr(l)=temp
          temp=brr(l+1)
          brr(l+1)=brr(l)
          brr(l)=temp
        endif
        i=l+1
        j=ir
        a=arr(l)
        b=brr(l)
3       continue
          i=i+1
        if(arr(i).lt.a)goto 3
4       continue
          j=j-1
        if(arr(j).gt.a)goto 4
        if(j.lt.i)goto 5
        temp=arr(i)
        arr(i)=arr(j)
        arr(j)=temp
        temp=brr(i)
        brr(i)=brr(j)
        brr(j)=temp
        goto 3
5       arr(l)=arr(j)
        arr(j)=a
        brr(l)=brr(j)
        brr(j)=b
        jstack=jstack+2
        if(jstack.gt.NSTACK)pause 'NSTACK too small in sort2'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 41m.

